@@ -16,11 +16,9 @@ if(is.null(family$multivariate)){
   Mu<-family$linkinv(Eta)
   SigmaInv <- 1/family$variance(Mu)
 }else{
-  Eta_cat <- matrix(Eta, byrow = TRUE, ncol = K)
-  Mu_cat <- family$linkinv(Eta_cat)
-  D <- family$deriv.mat(Mu_cat)
-  SigmaInv <- family$SigmaInv(Mu_cat)
-  Mu <- c(t(Mu_cat))
+  Mu <- family$linkinv(Eta, K)
+  D <- family$deriv.mat(Eta, K)
+  SigmaInv <- family$SigmaInv(Mu, K)
 }
 
 if(print.iter.final)
@@ -59,14 +57,15 @@ if(is.null(family$multivariate)){
   score_vec <- t(Z_alles)%*%((y-Mu)*D*SigmaInv)-P1%*%Delta[1,]
   F_gross<-t(Z_alles)%*%(Z_alles*D*SigmaInv*D)+P1
 }else{
-  score_vec <- t(Z_alles)%*%(D%*%(SigmaInv%*%(y-Mu)))-P1%*%Delta[1,]
-  F_gross<-t(Z_alles)%*%(D%*%(SigmaInv%*%(t(D)%*%Z_alles)))+P1
+  score_vec<-RcppEigenProd1(Z_alles, D, SigmaInv, y, Mu)-P1%*%Delta[1,]
+  W_opt <- RcppEigenProd2(D, SigmaInv)
+  F_gross <- t(Z_alles)%*%(W_opt%*%Z_alles)+P1
 }
 
 InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
-if(class(InvFisher)=="try-error")
+if(inherits(InvFisher, "try-error"))
 InvFisher<-try(solve(F_gross),silent=TRUE)  
-if(class(InvFisher)=="try-error")
+if(inherits(InvFisher, "try-error"))
 stop("Fisher matrix not invertible")
 
 half.index<-0
@@ -90,11 +89,9 @@ if(is.null(family$multivariate)){
   Mu<-family$linkinv(Eta)
   SigmaInv <- 1/family$variance(Mu)
 }else{
-  Eta_cat <- matrix(Eta, byrow = TRUE, ncol = K)
-  Mu_cat <- family$linkinv(Eta_cat)
-  D <- family$deriv.mat(Mu_cat)
-  SigmaInv <- family$SigmaInv(Mu_cat)
-  Mu <- c(t(Mu_cat))
+  Mu <- family$linkinv(Eta, K)
+  D <- family$deriv.mat(Eta, K)
+  SigmaInv <- family$SigmaInv(Mu, K)
 }
 
 if (method=="EM")
@@ -103,12 +100,13 @@ if (method=="EM")
       D <- drop(D);SigmaInv <- drop(SigmaInv)
       F_gross<-t(Z_alles)%*%(Z_alles*D*SigmaInv*D)+P1.old
     }else{
-      F_gross<-t(Z_alles)%*%(D%*%(SigmaInv%*%(t(D)%*%Z_alles)))+P1.old
+      W_opt <- RcppEigenProd2(D, SigmaInv)
+      F_gross <- t(Z_alles)%*%(W_opt%*%Z_alles)+P1.old
     }
   InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
-  if(class(InvFisher)=="try-error")
+  if(inherits(InvFisher, "try-error"))
     InvFisher<-try(solve(F_gross),silent=TRUE)  
-  if(class(InvFisher)=="try-error")
+  if(inherits(InvFisher, "try-error"))
   {
     half.index<-half.index+1  
   }else{
@@ -157,7 +155,7 @@ diag(Q1)<-(optim.obj$par[1:s])
       Q1[lower.tri(Q1)]<-((0.5)^ttt)*Q1[lower.tri(Q1)]
       Q1[upper.tri(Q1)]<-((0.5)^ttt)*Q1[upper.tri(Q1)]
       Q_solvetest<-try(solve(Q1))
-         if(all (eigen(Q1)$values>0) & class(Q_solvetest)!="try-error")
+         if(all (eigen(Q1)$values>0) & !inherits(Q_solvetest, "try-error"))
          break
       }
 }}
@@ -181,14 +179,15 @@ if(is.null(family$multivariate)){
   score_vec<-t(Z_alles)%*%((y-Mu)*D*SigmaInv)-P1%*%Delta[1,]
   F_gross<-t(Z_alles)%*%(Z_alles*D*SigmaInv*D)+P1
 }else{
-  score_vec <- t(Z_alles)%*%(D%*%(SigmaInv%*%(y-Mu)))-P1%*%Delta[1,]
-  F_gross<-t(Z_alles)%*%(D%*%(SigmaInv%*%(t(D)%*%Z_alles)))+P1
+  score_vec<-RcppEigenProd1(Z_alles, D, SigmaInv, y, Mu)-P1%*%Delta[1,]
+  W_opt <- RcppEigenProd2(D, SigmaInv)
+  F_gross <- t(Z_alles)%*%(W_opt%*%Z_alles)+P1
 }
 
 InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
-if(class(InvFisher)=="try-error")
+if(inherits(InvFisher, "try-error"))
   InvFisher<-try(solve(F_gross),silent=TRUE)  
-if(class(InvFisher)=="try-error")
+if(inherits(InvFisher, "try-error"))
 {
   half.index<-half.index+1  
 }else{
@@ -240,11 +239,9 @@ if(is.null(family$multivariate)){
   Mu<-family$linkinv(Eta)
   SigmaInv <- 1/family$variance(Mu)
 }else{
-  Eta_cat <- matrix(Eta, byrow = TRUE, ncol = K)
-  Mu_cat <- family$linkinv(Eta_cat)
-  D <- family$deriv.mat(Mu_cat)
-  SigmaInv <- family$SigmaInv(Mu_cat)
-  Mu <- c(t(Mu_cat))
+  Mu <- family$linkinv(Eta, K)
+  D <- family$deriv.mat(Eta, K)
+  SigmaInv <- family$SigmaInv(Mu, K)
 }
 
 if (method=="EM")
@@ -253,12 +250,13 @@ if (method=="EM")
     D <- drop(D);SigmaInv <- drop(SigmaInv)
     F_gross<-t(Z_alles)%*%(Z_alles*D*SigmaInv*D)+P1.old
   }else{
-    F_gross<-t(Z_alles)%*%(D%*%(SigmaInv%*%(t(D)%*%Z_alles)))+P1.old
+    W_opt <- RcppEigenProd2(D, SigmaInv)
+    F_gross <- t(Z_alles)%*%(W_opt%*%Z_alles)+P1.old
   }
   InvFisher2<-try(chol2inv(chol(F_gross)),silent=TRUE)
-  if(class(InvFisher2)=="try-error")
+  if(inherits(InvFisher2, "try-error"))
     InvFisher2<-try(solve(F_gross),silent=TRUE)  
-  if(class(InvFisher2)=="try-error")
+  if(inherits(InvFisher2, "try-error"))
   {
     half.index<-half.index+1  
   }else{
@@ -315,7 +313,7 @@ for (ttt in 0:100)
       Q1[lower.tri(Q1)]<-((0.5)^ttt)*Q1[lower.tri(Q1)]
       Q1[upper.tri(Q1)]<-((0.5)^ttt)*Q1[upper.tri(Q1)]
        Q_solvetest<-try(solve(Q1))
-         if(all (eigen(Q1)$values>0) & class(Q_solvetest)!="try-error")
+         if(all (eigen(Q1)$values>0) & !inherits(Q_solvetest, "try-error"))
          break
       }
 }}
@@ -339,14 +337,15 @@ if(is.null(family$multivariate)){
   score_vec<-t(Z_alles)%*%((y-Mu)*D*SigmaInv)-P1%*%Delta[l,]
   F_gross<-t(Z_alles)%*%(Z_alles*D*SigmaInv*D)+P1
 }else{
-  score_vec <- t(Z_alles)%*%(D%*%(SigmaInv%*%(y-Mu)))-P1%*%Delta[l,]
-  F_gross<-t(Z_alles)%*%(D%*%(SigmaInv%*%(t(D)%*%Z_alles)))+P1
+  score_vec<-RcppEigenProd1(Z_alles, D, SigmaInv, y, Mu)-P1%*%Delta[l,]
+  W_opt <- RcppEigenProd2(D, SigmaInv)
+  F_gross <- t(Z_alles)%*%(W_opt%*%Z_alles)+P1
 }
   
 InvFisher<-try(chol2inv(chol(F_gross)),silent=TRUE)
-if(class(InvFisher)=="try-error")
+if(inherits(InvFisher, "try-error"))
   InvFisher<-try(solve(F_gross),silent=TRUE)  
-if(class(InvFisher)=="try-error")
+if(inherits(InvFisher, "try-error"))
 {
   half.index<-half.index+1  
 }else{
@@ -372,9 +371,8 @@ if(is.null(family$multivariate)){
   W_opt <- D*SigmaInv*D
   FinalHat<-(Z_alles*sqrt(W_opt))%*%InvFisher%*%t(Z_alles*sqrt(W_opt))
 }else{
-  W_opt <- D%*%(SigmaInv%*%t(D))
-  W_inv_t <- chol(W_opt)
-  FinalHat<-W_inv_t%*%(Z_alles%*%(InvFisher%*%(t(Z_alles)%*%t(W_inv_t))))
+  W_inv_t <- RcppEigenSpChol(W_opt)
+  FinalHat <- RcppEigenProd3(W_inv_t, Z_alles, InvFisher)
 }
 
 complexity<-sum(diag(FinalHat))
